<?php

/** \mainpage Stack2RSS Documentation
 *
 * \section introduction Introduction
 * Stack2RSS is a simple web application that creates an RSS feed given an
 * API route.
 *
 * \section usage Usage
 * Using Stack2RSS is relatively simple. The general format of the URL for a feed looks something like this:
 * <pre>/ <b>{Stack Exchange Site Domain}</b> / <b>{API route}</b> ? <b>{Parameters}</b></pre>
 * There are a couple things to note here:
 * - The domain is of the form <kbd>site.com</kbd> (examples: <kbd>stackoverflow.com</kbd>, <kbd>apple.stackexchange.com</kbd>, <kbd>stackauth.com</kbd>).
 * - You do not need to specify an API key as Stack2RSS will use its own key.
 * - You can use v1.0 methods by passing the 'api_version=1.0' query string parameter.
 *
 * \section summary Summary of the Process
 * -# Fetch and process all parameters
 * -# Make API request to given method
 * -# Convert results returned to HTML
 * -# Create RSS-compliant feed from items
 * -# Echo it to the client
 *  
 * \section structure Code Structure
 * The code is basically divided into two sections:
 * - the Stack2RSS class that is actually responsible for the requests / parameter parsing / etc.
 * - a bunch of classes that derive from BaseConverter which convert one particular type of item (such as 'user' or 'question') into an HTML representation
 *
 * \section imp_classes Important Classes
 * - Stack2RSS
 * - Converter
 *
 * \section conv_classes Conversion Classes
 * - AnswerConverter
 * - APISiteConverter
 * - AssociatedUserConverter
 * - BadgeConverter
 * - CommentConverter
 * - ItemConverter
 * - PostTimelineConverter
 * - PrivilegeConverter
 * - QuestionConverter
 * - RepChangeConverter
 * - RevisionConverter
 * - StatisticsConverter
 * - TagSynonymConverter
 * - TagWikiConverter
 * - TagConverter
 * - TopTagConverter
 * - TopUserConverter
 * - UserTimelineConverter
 * - UserConverter
 */
 
/// \file feed.php Contains the main class, Stack2RSS.

// Load our configuration file
require_once 'config.inc';

// Include the different types of conversion classes
require_once 'converters/answer_converter.php';
require_once 'converters/api_site_converter.php';
require_once 'converters/associated_user_converter.php';
require_once 'converters/badge_converter.php';
require_once 'converters/comment_converter.php';
require_once 'converters/item_converter.php';
require_once 'converters/post_timeline_converter.php';
require_once 'converters/privilege_converter.php';
require_once 'converters/question_converter.php';
require_once 'converters/rep_change_converter.php';
require_once 'converters/revision_converter.php';
require_once 'converters/statistics_converter.php';
require_once 'converters/tag_synonym_converter.php';
require_once 'converters/tag_wiki_converter.php';
require_once 'converters/tag_converter.php';
require_once 'converters/top_tag_converter.php';
require_once 'converters/top_user_converter.php';
require_once 'converters/user_timeline_converter.php';
require_once 'converters/user_converter.php';

/// Fetches, parses, and processes API requests
/**
 * Stack2RSS is responsible for parsing the parameters it is passed, fetching the API requests, and finally processing them by creating an instance of the appropriate conversion class and passing along the data.
 */
class Stack2RSS
{
    /// Returns an error to the client and exits
    /**
     * \param[in] $error_str a string representation of the error
     */
    private function OutputError($error_str, $status_line='404 Not Found')
    {
        // Outputting an error that consists of a 404 status code
        // followed by sending the message
        header("HTTP/1.1 $status_line");
        
        $error_title = 'There was an error processing this page.';
        $error_desc  = $error_str;
        require 'error.php';
        
        // Now simply end program execution
        exit;
    }
    
    /// Fetches the raw JSON data from the API for the given URL
    /**
     * <i>Note: this method uses PHP's cURL methods.</i>
     * \param[in] $url the URL of the API method to query
     * \return an array object with the JSON-decoded data
     */
    private function FetchData($url)
    {
        $ch = curl_init();
        
        curl_setopt($ch,CURLOPT_URL,$url);
        curl_setopt($ch,CURLOPT_HEADER,0);
        curl_setopt($ch,CURLOPT_USERAGENT,"Stack2RSS/1.0");
        curl_setopt($ch,CURLOPT_ENCODING,'gzip');  // Needed by API
        curl_setopt($ch,CURLOPT_RETURNTRANSFER,1);
        
        $data = curl_exec($ch);
        
        // Check for an error
        if($data === FALSE)
            $this->OutputError('There was an error retrieving the data from the API servers.', '500 Internal Server Error');
        
        curl_close($ch);
        
        // Now decode the data and check for an API error.
        $decoded_data = json_decode($data, TRUE);
        
        if(isset($decoded_data['error']))
            $this->OutputError("Error reported by the API:<br /><br /><kbd>{$decoded_data['error']['message']}</kbd>");
        
        return $decoded_data;
    }
    
    /// Gathers information from parameters and generates a URL
    /**
     * This method is responsible for finding all of the parameters that have been passed to the script including the site and constructing the URL that will be passed to FetchData().
     * \return the generated URL
     */
    private function GenerateURLFromParameters()
    {
        $site    = 'stackoverflow.com'; // use SO by default
        $method  = 'questions';         // '/questions' method by default
        $version = '1.1';               // the API version to use

        // This array contains all of the parameters that will be
        // included in the API request. We always include our API key
        // at a minimum.
        $parameters = array('key=Gbn011-pzEi4tPK3BwbYIg');
        
        // Loop through all of the $_GET variables.
        foreach($_GET as $key => $value)
        {
            // All of the parameters are added to the list except for two
            // special parameters: site and method
            if($key == 'site')
                $site = $value;
            elseif($key == 'method')
                $method = $value;
            elseif($key == 'api_version')
                $version = $value;
            else
                $parameters[] = urlencode($key) . '=' . urlencode($value);
        }
        
        // If no method was supplied, we redirect the user to the home
        // page where they can discover why it makes no sense to do such a thing.
        if($method == '')
        {
            // Output an error
            require 'error.php';
            exit;
        }
        
        // Make sure the site has a TLD domain in it (for backward compatibility,
        // we insert '.com' if there isn't one. Also modify the $_GET variable to include it.
        if(substr($site, -4) != '.com')
        {
            $site .= '.com';
            $_GET['site'] .= '.com';
        }
        
        // Unless the site is stackauth, we prepend 'api.' to the URL
        $prepend = ($site == 'stackauth.com')?'':'api.';
        
        // Now construct the URL to return
        return "http://{$prepend}{$site}/{$version}/{$method}?" . implode('&', $parameters);
    }
    
    /// Determines the correct conversion class to instantiate and uses it
    /**
     * \param[in] $json_data the JSON-encoded data returned by the API
     * \return an array of items with HTML for title, description, etc.
     */
    private function RunConversion($json_data)
    {
        // Find the array item in the response
        foreach($json_data as $key => $value)
            if(is_array($value))
            {
                // Now check to see what type of key was used
                switch($key)
                {
                    case 'answers':
                        $converter = new AnswerConverter();
                        break;
                    case 'api_sites':
                        $converter = new APISiteConverter();
                        break;
                    case 'associated_users':
                        $converter = new AssociatedUserConverter();
                        break;
                    case 'badges':
                        $converter = new BadgeConverter();
                        break;
                    case 'comments':
                        $converter = new CommentConverter();
                        break;
                    case 'items':
                        $converter = new ItemConverter();
                        break;
                    case 'post_timelines':
                        $converter = new PostTimelineConverter();
                        break;
                    case 'privileges':
                        $converter = new PrivilegeConverter();
                        break;
                    case 'questions':
                        $converter = new QuestionConverter();
                        break;
                    case 'rep_changes':
                        $converter = new RepChangeConverter();
                        break;
                    case 'revisions':
                        $converter = new RevisionConverter();
                        break;
                    case 'statistics':
                        $converter = new StatisticsConverter();
                        break;
                    case 'tag_synonyms':
                        $converter = new TagSynonymConverter();
                        break;
                    case 'tag_wikis':
                        $converter = new TagWikiConverter();
                        break;
                    case 'tags':
                        $converter = new TagConverter();
                        break;
                    case 'top_tags':
                        $converter = new TopTagConverter();
                        break;
                    case 'top_users':
                        $converter = new TopUserConverter();
                        break;
                    case 'user_timelines':
                        $converter = new UserTimelineConverter();
                        break;
                    case 'users':
                        $converter = new UserConverter();
                        break;
                    default:
                        // The response is a type we do not recognize
                        $this->OutputError('Unknown data type returned by the API.', '500 Internal Server Error');
                        break;
                }
                
                // Perform the actual conversion and return the results
                return $converter->PerformConversion($value);
            }
        
        // If we reach this line, then no array object was found
        $this->OutputError('The API response did not contain an array.', '500 Internal Server Error');
    }
    
    /// Processes the feed
    /**
     * <i>Note: this is the only public function in this class - it basically just wraps the other private functions, tying them all together.</i>
     */
    public function ProcessFeed()
    {
        $url        = $this->GenerateURLFromParameters();
        $json_data  = $this->FetchData($url);
        $html_items = $this->RunConversion($json_data);
        
        // Now output the results
        header('Content-type: application/rss+xml');
        echo $this->CreateFeedData($html_items, $url);
    }
    
    /// Creates the actual RSS feed data
    /**
     * This function returns the XML for the RSS feed including all of the metadata for the feed.
     * \param[in] $items_data an array that contians the data (title, HTML) for each item
     * \param[in] $feed_url   the URL for the API request
     * \return the actual feed data itself
     */
    private function CreateFeedData($items_data, $feed_url)
    {
        // Gather the data we need for creating the XML
        $encoded_feed_url = htmlentities($feed_url);
        $build_date       = date(DATE_RSS);
        
        // Now create the XML that combines the feed items together
        $feed_item_html = '';
        foreach($items_data as $item_data)
        {
$feed_item_html .= <<<EOD
    <item>
      <title><![CDATA[{$item_data['title']}]]></title>
      <link>{$item_data['link']}</link>
      <description><![CDATA[{$item_data['description']}]]></description>
      <pubDate>{$item_data['date']}</pubDate>
    </item>
EOD;
        }
        
        // Now return that data
return <<<EOD
<?xml version="1.0" encoding="UTF-8" ?>
<rss version="2.0" xmlns:dc="http://purl.org/dc/elements/1.1/">
  <channel>
    <title>Stack2RSS Custom Feed</title>
    <link>{$encoded_feed_url}</link>
    <description>This feed contains the result of the API URL {$encoded_feed_url}.</description>
    <language>en</language>
    <generator>PHP 5.2</generator>
    <lastBuildDate>{$build_date}</lastBuildDate>
{$feed_item_html}
  </channel>
</rss>
EOD;
    }
}

// Create an instance of Stack2RSS.
$feed = new Stack2RSS();
$feed->ProcessFeed();

?>
